<?php

namespace Drupal\entity_fallback_value\Service;

use Drupal\Core\Access\AccessibleInterface;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\entity_fallback_value\PluginManager\EntityFallbackValuePluginManager;
use Drupal\entity_fallback_value\Traits\AccessNestedFieldsTrait;
use Twig\Extension\AbstractExtension;
use Twig\Extension\ExtensionInterface;
use Twig\TwigFunction;

/**
 * Service allowing to get fallback values for a content entity.
 *
 * @package Drupal\entity_fallback_value\Service
 */
class EntityFallbackValueManager extends AbstractExtension implements ExtensionInterface {
  use AccessNestedFieldsTrait;

  /**
   * EntityFallbackValue Plugin Manager.
   *
   * @var \Drupal\entity_fallback_value\PluginManager\EntityFallbackValuePluginManager
   */
  protected $fallbackValuesPluginmanager;

  /**
   * Current langcode.
   *
   * @var string
   */
  protected $currentLangCode;

  /**
   * TwigEntityFallbackValueTools constructor.
   *
   * @param \Drupal\entity_fallback_value\PluginManager\EntityFallbackValuePluginManager $fallbackValuesPluginManager
   *   The fallbackValuesPluginManager.
   * @param \Drupal\Core\Language\LanguageManagerInterface $languageManager
   *   The Language manager.
   * @param \Drupal\Core\Entity\EntityRepositoryInterface $entityRepository
   *   The entity repository.
   */
  public function __construct(EntityFallbackValuePluginManager $fallbackValuesPluginManager, LanguageManagerInterface $languageManager, EntityRepositoryInterface $entityRepository) {
    $this->fallbackValuesPluginmanager = $fallbackValuesPluginManager;
    $this->currentLangCode = $languageManager->getCurrentLanguage()->getId();
    $this->setEntityRepository($entityRepository);
  }

  /**
   * Returns the list f available functions.
   *
   * @return array
   *   List of functions.
   */
  public function getFunctions() {
    return [
      new TwigFunction('getEntityFallbackValues', [
        $this,
        'getEntityFallbackValues',
      ]),
    ];
  }

  /**
   * Return the fallback values.
   *
   * @param \Drupal\Core\Access\AccessibleInterface|null $contentEntity
   *   The content entity.
   * @param mixed|null $keys
   *   THe keys.
   * @param mixed|null $definitions
   *   THe list of custom defintions.
   *
   * @return array
   *   The values.
   */
  public function getEntityFallbackValues(AccessibleInterface $contentEntity = NULL, $keys = NULL, $definitions = NULL) {
    $values = [];
    if ($contentEntity && $contentEntity instanceof ContentEntityInterface) {
      if (is_array($definitions)) {
        // Load fallback values from custom definitions.
        $values = $this->getEntityCustomFallbackValues($contentEntity, $definitions, $keys);
      }
      else {
        // Load fallback values from default (plugin) definitions.
        $values = $this->getEntityDefaultFallbackValues($contentEntity, $keys);
      }
    }

    return $values;
  }

  /**
   * Return the fallback values according to plugins.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $contentEntity
   *   The content entity.
   * @param mixed|null $keys
   *   The keys.
   *
   * @return array
   *   The values.
   */
  public function getEntityDefaultFallbackValues(ContentEntityInterface $contentEntity, $keys = NULL) {
    $values = [];
    if (isset($contentEntity->fallbackValues)) {
      $values = $contentEntity->fallbackValues;
    }
    elseif ($plugins = $this->fallbackValuesPluginmanager->getInstancesByContentEntity($contentEntity)) {
      $contentEntity->fallbackValues = [];
      if ($contentEntity->hasTranslation($this->currentLangCode)) {
        foreach ($plugins as $plugin) {
          $contentEntity->fallbackValues = array_merge(
            $contentEntity->fallbackValues,
            $plugin->getEntityFallbackValues($contentEntity->getTranslation($this->currentLangCode)));
        }
      }

      $values = $contentEntity->fallbackValues;
    }

    return $this->getOnlyKeys($values, $keys);
  }

  /**
   * Return the fallback values according to definition.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $contentEntity
   *   The content entity.
   * @param array $definitions
   *   THe definitions.
   * @param mixed|null $keys
   *   THe keys.
   *
   * @return array
   *   The values.
   */
  public function getEntityCustomFallbackValues(ContentEntityInterface $contentEntity, array $definitions = [], $keys = NULL) {
    $values = $this->getEntityFallbackValuesFromDefintions($contentEntity, $definitions);

    return $this->getOnlyKeys($values, $keys);
  }

  /**
   * REturn only the wanted values.
   *
   * @param array $values
   *   THe values.
   * @param mixed|null $keys
   *   The wanted keys.
   *
   * @return array
   *   THe filtered values.
   */
  protected function getOnlyKeys(array $values, $keys = NULL) {
    if ($keys) {
      $keys = is_array($keys) ? $keys : [$keys];
      $values = array_intersect_key($values, array_flip($keys));
    }

    return $values;
  }

}
