<?php

namespace Drupal\entity_fallback_value\Traits;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\Field\EntityReferenceFieldItemListInterface;
use Drupal\Core\Field\FieldItemInterface;
use Drupal\Core\TypedData\ListInterface;
use Drupal\Core\TypedData\PrimitiveBase;

/**
 * Trait to accesss nested fields.
 *
 * @package Drupal\entity_fallback_value\Traits
 */
trait AccessNestedFieldsTrait {

  /**
   * Entity Repository.
   *
   * @var \Drupal\Core\Entity\EntityDisplayRepositoryInterface
   */
  protected EntityRepositoryInterface $entityRepository;

  /**
   * Set the entityRepository.
   *
   * @param \Drupal\Core\Entity\EntityRepositoryInterface $entityRepository
   *   The entity repository.
   */
  public function setEntityRepository(EntityRepositoryInterface $entityRepository) {
    $this->entityRepository = $entityRepository;
  }

  /**
   * Return the entity fallback values.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface|null $contentEntity
   *   The content.
   * @param array $definitions
   *   THe list of definitions.
   *
   * @return array
   *   The values.
   */
  public function getEntityFallbackValuesFromDefintions(ContentEntityInterface $contentEntity = NULL, array $definitions = []) {
    $values = [];
    foreach ($definitions as $key => $definition) {
      $values[$key] = $this->getFallbackValue($contentEntity, $definition);
    }

    return $values;
  }

  /**
   * Return the list of fallback values.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $contentEntity
   *   The content entity.
   * @param array $fallbackDefinition
   *   The fallback definitions.
   *
   * @return \Drupal\Core\Entity\EntityInterface|\Drupal\Core\Entity\EntityInterface[]|\Drupal\Core\TypedData\TypedDataInterface|false|mixed|null
   *   The list of fallback values.
   */
  protected function getFallbackValue(ContentEntityInterface $contentEntity, array $fallbackDefinition) {
    foreach ($fallbackDefinition as $definition) {
      try {
        $value = $this->getDefinitionValue($contentEntity, $definition);
        if (!empty($value)) {
          return $value;
        }
      }
      catch (\Exception $e) {
        // Mute exception...
      }
    }

    return NULL;
  }

  /**
   * Return the value for a nested field definition.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $contentEntity
   *   The content entity.
   * @param string $definition
   *   The definition.
   *
   * @return \Drupal\Core\Entity\EntityInterface|\Drupal\Core\Entity\EntityInterface[]|\Drupal\Core\TypedData\TypedDataInterface|false|mixed|null
   *   THe value.
   *
   * @throws \Drupal\Core\TypedData\Exception\MissingDataException
   */
  protected function getDefinitionValue(ContentEntityInterface $contentEntity, $definition) {
    $nestedField = explode('.', $definition);

    return $this->getFieldValue($contentEntity, $nestedField);
  }

  /**
   * Return the field value.
   *
   * @param mixed $parentEntity
   *   The parent entity.
   * @param array $nestedFields
   *   The list of nested fields.
   *
   * @return \Drupal\Core\Entity\EntityInterface|\Drupal\Core\Entity\EntityInterface[]|\Drupal\Core\TypedData\TypedDataInterface|false|mixed|null
   *   THe value.
   *
   * @throws \Drupal\Core\TypedData\Exception\MissingDataException
   */
  protected function getFieldValue($parentEntity, array $nestedFields = []) {
    if (count($nestedFields) > 0) {
      $field = reset($nestedFields);
      $nextFields = array_slice($nestedFields, 1);
      if ($parentField = $this->getFieldItemValue($parentEntity->get($field))) {
        if (count($nextFields)) {
          return $this->getFieldValue($parentField, $nextFields);
        }
        else {
          return $this->getFieldItemValue($parentField);
        }
      }
    }

    return NULL;
  }

  /**
   * Get field item value.
   *
   * @param mixed $fieldItem
   *   The field item.
   *
   * @return \Drupal\Core\Entity\EntityInterface|\Drupal\Core\Entity\EntityInterface[]|\Drupal\Core\TypedData\TypedDataInterface|false|mixed|null
   *   The value.
   *
   * @throws \Drupal\Core\TypedData\Exception\MissingDataException
   */
  protected function getFieldItemValue($fieldItem) {
    if ($fieldItem instanceof EntityReferenceFieldItemListInterface) {
      $entities = $fieldItem->referencedEntities();
      if ($this->entityRepository && $this->entityRepository instanceof EntityRepositoryInterface) {
        foreach ($entities as $key => $entity) {
          $entities[$key] = $this->entityRepository->getTranslationFromContext($entity);
        }
      }

      return reset($entities);
    }
    if ($fieldItem instanceof ListInterface) {
      return $fieldItem->first();
    }
    if ($fieldItem instanceof EntityReferenceFieldItemListInterface) {
      return $fieldItem->referencedEntities();
    }
    if ($fieldItem instanceof FieldItemInterface) {
      $value = $fieldItem->getValue();

      return $value;
    }
    if ($fieldItem instanceof PrimitiveBase) {
      return $fieldItem->getValue();
    }

    return $fieldItem;
  }

}
